/****************************************************************************
** Meta object code from reading C++ file 'replacebar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../controls/replacebar.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'replacebar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10ReplaceBarE_t {};
} // unnamed namespace

template <> constexpr inline auto ReplaceBar::qt_create_metaobjectdata<qt_meta_tag_ZN10ReplaceBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ReplaceBar",
        "pressEsc",
        "",
        "replaceNext",
        "file",
        "replaceText",
        "withText",
        "replaceSkip",
        "keyword",
        "replaceRest",
        "replaceAll",
        "beforeReplace",
        "_",
        "backToPosition",
        "row",
        "column",
        "scrollOffset",
        "removeSearchKeyword",
        "updateSearchKeyword",
        "sigReplacebarClose",
        "change",
        "replaceClose",
        "handleContentChanged",
        "handleReplaceAll",
        "handleReplaceNext",
        "handleReplaceRest",
        "handleSkip",
        "updateSizeMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pressEsc'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replaceNext'
        QtMocHelpers::SignalData<void(QString, QString, QString)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'replaceSkip'
        QtMocHelpers::SignalData<void(QString, QString)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'replaceRest'
        QtMocHelpers::SignalData<void(QString, QString)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'replaceAll'
        QtMocHelpers::SignalData<void(QString, QString)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'beforeReplace'
        QtMocHelpers::SignalData<void(QString)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'backToPosition'
        QtMocHelpers::SignalData<void(QString, int, int, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::Int, 14 }, { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'removeSearchKeyword'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateSearchKeyword'
        QtMocHelpers::SignalData<void(QString, QString)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'sigReplacebarClose'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'change'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'replaceClose'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleContentChanged'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleReplaceAll'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleReplaceNext'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleReplaceRest'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSkip'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSizeMode'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ReplaceBar, qt_meta_tag_ZN10ReplaceBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ReplaceBar::staticMetaObject = { {
    QMetaObject::SuperData::link<DFloatingWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ReplaceBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ReplaceBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10ReplaceBarE_t>.metaTypes,
    nullptr
} };

void ReplaceBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ReplaceBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pressEsc(); break;
        case 1: _t->replaceNext((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->replaceSkip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->replaceRest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->replaceAll((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->beforeReplace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->backToPosition((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 7: _t->removeSearchKeyword(); break;
        case 8: _t->updateSearchKeyword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->sigReplacebarClose(); break;
        case 10: _t->change(); break;
        case 11: _t->replaceClose(); break;
        case 12: _t->handleContentChanged(); break;
        case 13: _t->handleReplaceAll(); break;
        case 14: _t->handleReplaceNext(); break;
        case 15: _t->handleReplaceRest(); break;
        case 16: _t->handleSkip(); break;
        case 17: _t->updateSizeMode(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)()>(_a, &ReplaceBar::pressEsc, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)(QString , QString , QString )>(_a, &ReplaceBar::replaceNext, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)(QString , QString )>(_a, &ReplaceBar::replaceSkip, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)(QString , QString )>(_a, &ReplaceBar::replaceRest, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)(QString , QString )>(_a, &ReplaceBar::replaceAll, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)(QString )>(_a, &ReplaceBar::beforeReplace, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)(QString , int , int , int )>(_a, &ReplaceBar::backToPosition, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)()>(_a, &ReplaceBar::removeSearchKeyword, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)(QString , QString )>(_a, &ReplaceBar::updateSearchKeyword, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReplaceBar::*)()>(_a, &ReplaceBar::sigReplacebarClose, 9))
            return;
    }
}

const QMetaObject *ReplaceBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ReplaceBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ReplaceBarE_t>.strings))
        return static_cast<void*>(this);
    return DFloatingWidget::qt_metacast(_clname);
}

int ReplaceBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DFloatingWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void ReplaceBar::pressEsc()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ReplaceBar::replaceNext(QString _t1, QString _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void ReplaceBar::replaceSkip(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void ReplaceBar::replaceRest(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void ReplaceBar::replaceAll(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void ReplaceBar::beforeReplace(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ReplaceBar::backToPosition(QString _t1, int _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 7
void ReplaceBar::removeSearchKeyword()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ReplaceBar::updateSearchKeyword(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void ReplaceBar::sigReplacebarClose()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
