/****************************************************************************
** Meta object code from reading C++ file 'linebar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../controls/linebar.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'linebar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7LineBarE_t {};
} // unnamed namespace

template <> constexpr inline auto LineBar::qt_create_metaobjectdata<qt_meta_tag_ZN7LineBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LineBar",
        "contentChanged",
        "",
        "focusOut",
        "pressAltEnter",
        "pressCtrlEnter",
        "pressEnter",
        "pressMetaEnter",
        "signal_sentText",
        "t",
        "handleTextChangeTimer",
        "handleTextChanged",
        "str",
        "sendText"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'contentChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'focusOut'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pressAltEnter'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pressCtrlEnter'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pressEnter'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pressMetaEnter'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signal_sentText'
        QtMocHelpers::SignalData<void(QString)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'handleTextChangeTimer'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'handleTextChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'sendText'
        QtMocHelpers::SlotData<void(QString)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LineBar, qt_meta_tag_ZN7LineBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LineBar::staticMetaObject = { {
    QMetaObject::SuperData::link<DLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LineBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LineBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7LineBarE_t>.metaTypes,
    nullptr
} };

void LineBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LineBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contentChanged(); break;
        case 1: _t->focusOut(); break;
        case 2: _t->pressAltEnter(); break;
        case 3: _t->pressCtrlEnter(); break;
        case 4: _t->pressEnter(); break;
        case 5: _t->pressMetaEnter(); break;
        case 6: _t->signal_sentText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->handleTextChangeTimer(); break;
        case 8: _t->handleTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->handleTextChanged(); break;
        case 10: _t->sendText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LineBar::*)()>(_a, &LineBar::contentChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LineBar::*)()>(_a, &LineBar::focusOut, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LineBar::*)()>(_a, &LineBar::pressAltEnter, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LineBar::*)()>(_a, &LineBar::pressCtrlEnter, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LineBar::*)()>(_a, &LineBar::pressEnter, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LineBar::*)()>(_a, &LineBar::pressMetaEnter, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (LineBar::*)(QString )>(_a, &LineBar::signal_sentText, 6))
            return;
    }
}

const QMetaObject *LineBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LineBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LineBarE_t>.strings))
        return static_cast<void*>(this);
    return DLineEdit::qt_metacast(_clname);
}

int LineBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void LineBar::contentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LineBar::focusOut()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void LineBar::pressAltEnter()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void LineBar::pressCtrlEnter()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void LineBar::pressEnter()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void LineBar::pressMetaEnter()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void LineBar::signal_sentText(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
