/****************************************************************************
** Meta object code from reading C++ file 'startmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../startmanager.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'startmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12StartManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto StartManager::qt_create_metaobjectdata<qt_meta_tag_ZN12StartManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StartManager",
        "D-Bus Interface",
        "com.deepin.Editor",
        "openFilesInTab",
        "",
        "files",
        "openFilesInWindow",
        "createWindowFromWrapper",
        "tabName",
        "filePath",
        "qstrTruePath",
        "EditWrapper*",
        "buffer",
        "isModifyed",
        "loadTheme",
        "themeName",
        "createWindow",
        "Window*",
        "alwaysCenter",
        "initWindowPosition",
        "window",
        "popupExistTabs",
        "FileTabInfo",
        "info",
        "getFileTabInfo",
        "file",
        "slotCheckUnsaveTab",
        "closeAboutForWindow",
        "slotCreatNewwindow",
        "slotCloseWindow",
        "slotDelayBackupFile"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'openFilesInTab'
        QtMocHelpers::SlotData<void(QStringList)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'openFilesInWindow'
        QtMocHelpers::SlotData<void(QStringList)>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'createWindowFromWrapper'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, EditWrapper *, bool)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { 0x80000000 | 11, 12 },
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'loadTheme'
        QtMocHelpers::SlotData<void(const QString &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'createWindow'
        QtMocHelpers::SlotData<Window *(bool)>(16, 4, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'createWindow'
        QtMocHelpers::SlotData<Window *()>(16, 4, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 17),
        // Slot 'initWindowPosition'
        QtMocHelpers::SlotData<void(Window *, bool)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 20 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'initWindowPosition'
        QtMocHelpers::SlotData<void(Window *)>(19, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 17, 20 },
        }}),
        // Slot 'popupExistTabs'
        QtMocHelpers::SlotData<void(FileTabInfo)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'getFileTabInfo'
        QtMocHelpers::SlotData<FileTabInfo(QString)>(24, 4, QMC::AccessPublic, 0x80000000 | 22, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'slotCheckUnsaveTab'
        QtMocHelpers::SlotData<void()>(26, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closeAboutForWindow'
        QtMocHelpers::SlotData<void(Window *)>(27, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 20 },
        }}),
        // Slot 'slotCreatNewwindow'
        QtMocHelpers::SlotData<void()>(28, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCloseWindow'
        QtMocHelpers::SlotData<void()>(29, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDelayBackupFile'
        QtMocHelpers::SlotData<void()>(30, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<StartManager, qt_meta_tag_ZN12StartManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject StartManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StartManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StartManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12StartManagerE_t>.metaTypes,
    nullptr
} };

void StartManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StartManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->openFilesInTab((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 1: _t->openFilesInWindow((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->createWindowFromWrapper((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<EditWrapper*>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 3: _t->loadTheme((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: { Window* _r = _t->createWindow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Window**>(_a[0]) = std::move(_r); }  break;
        case 5: { Window* _r = _t->createWindow();
            if (_a[0]) *reinterpret_cast<Window**>(_a[0]) = std::move(_r); }  break;
        case 6: _t->initWindowPosition((*reinterpret_cast<std::add_pointer_t<Window*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->initWindowPosition((*reinterpret_cast<std::add_pointer_t<Window*>>(_a[1]))); break;
        case 8: _t->popupExistTabs((*reinterpret_cast<std::add_pointer_t<FileTabInfo>>(_a[1]))); break;
        case 9: { FileTabInfo _r = _t->getFileTabInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<FileTabInfo*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->slotCheckUnsaveTab(); break;
        case 11: _t->closeAboutForWindow((*reinterpret_cast<std::add_pointer_t<Window*>>(_a[1]))); break;
        case 12: _t->slotCreatNewwindow(); break;
        case 13: _t->slotCloseWindow(); break;
        case 14: _t->slotDelayBackupFile(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EditWrapper* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Window* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Window* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Window* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *StartManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StartManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StartManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StartManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
