/****************************************************************************
** Meta object code from reading C++ file 'settings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../common/settings.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'settings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13CustemBackendE_t {};
} // unnamed namespace

template <> constexpr inline auto CustemBackend::qt_create_metaobjectdata<qt_meta_tag_ZN13CustemBackendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustemBackend"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustemBackend, qt_meta_tag_ZN13CustemBackendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustemBackend::staticMetaObject = { {
    QMetaObject::SuperData::link<DSettingsBackend::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CustemBackendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CustemBackendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13CustemBackendE_t>.metaTypes,
    nullptr
} };

void CustemBackend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustemBackend *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustemBackend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustemBackend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CustemBackendE_t>.strings))
        return static_cast<void*>(this);
    return DSettingsBackend::qt_metacast(_clname);
}

int CustemBackend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DSettingsBackend::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8SettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto Settings::qt_create_metaobjectdata<qt_meta_tag_ZN8SettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Settings",
        "sigAdjustFont",
        "",
        "name",
        "sigAdjustFontSize",
        "fontSize",
        "sigAdjustTabSpaceNumber",
        "number",
        "sigAdjustWordWrap",
        "enable",
        "sigAdjustBookmark",
        "sigShowCodeFlodFlag",
        "sigShowBlankCharacter",
        "sigHightLightCurrentLine",
        "sigThemeChanged",
        "theme",
        "sigSetLineNumberShow",
        "bIsShow",
        "sigChangeWindowSize",
        "mode",
        "slotCustomshortcut",
        "key",
        "QVariant",
        "value",
        "slotsigAdjustFont",
        "slotsigAdjustFontSize",
        "slotsigAdjustWordWrap",
        "slotsigSetLineNumberShow",
        "slotsigAdjustBookmark",
        "slotsigShowCodeFlodFlag",
        "slotsigShowBlankCharacter",
        "slotsigHightLightCurrentLine",
        "slotsigAdjustTabSpaceNumber",
        "slotupdateAllKeysWithKeymap"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigAdjustFont'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'sigAdjustFontSize'
        QtMocHelpers::SignalData<void(qreal)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 5 },
        }}),
        // Signal 'sigAdjustTabSpaceNumber'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'sigAdjustWordWrap'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'sigAdjustBookmark'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'sigShowCodeFlodFlag'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'sigShowBlankCharacter'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'sigHightLightCurrentLine'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'sigThemeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'sigSetLineNumberShow'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'sigChangeWindowSize'
        QtMocHelpers::SignalData<void(QString)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'slotCustomshortcut'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigAdjustFont'
        QtMocHelpers::SlotData<void(QVariant)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigAdjustFontSize'
        QtMocHelpers::SlotData<void(QVariant)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigAdjustWordWrap'
        QtMocHelpers::SlotData<void(QVariant)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigSetLineNumberShow'
        QtMocHelpers::SlotData<void(QVariant)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigAdjustBookmark'
        QtMocHelpers::SlotData<void(QVariant)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigShowCodeFlodFlag'
        QtMocHelpers::SlotData<void(QVariant)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigShowBlankCharacter'
        QtMocHelpers::SlotData<void(QVariant)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigHightLightCurrentLine'
        QtMocHelpers::SlotData<void(QVariant)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotsigAdjustTabSpaceNumber'
        QtMocHelpers::SlotData<void(QVariant)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotupdateAllKeysWithKeymap'
        QtMocHelpers::SlotData<void(QVariant)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Settings, qt_meta_tag_ZN8SettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Settings::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8SettingsE_t>.metaTypes,
    nullptr
} };

void Settings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Settings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigAdjustFont((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->sigAdjustFontSize((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 2: _t->sigAdjustTabSpaceNumber((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->sigAdjustWordWrap((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->sigAdjustBookmark((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->sigShowCodeFlodFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->sigShowBlankCharacter((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->sigHightLightCurrentLine((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->sigThemeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->sigSetLineNumberShow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->sigChangeWindowSize((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotCustomshortcut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 12: _t->slotsigAdjustFont((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 13: _t->slotsigAdjustFontSize((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 14: _t->slotsigAdjustWordWrap((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 15: _t->slotsigSetLineNumberShow((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 16: _t->slotsigAdjustBookmark((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 17: _t->slotsigShowCodeFlodFlag((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 18: _t->slotsigShowBlankCharacter((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 19: _t->slotsigHightLightCurrentLine((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 20: _t->slotsigAdjustTabSpaceNumber((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 21: _t->slotupdateAllKeysWithKeymap((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(QString )>(_a, &Settings::sigAdjustFont, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(qreal )>(_a, &Settings::sigAdjustFontSize, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(int )>(_a, &Settings::sigAdjustTabSpaceNumber, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(bool )>(_a, &Settings::sigAdjustWordWrap, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(bool )>(_a, &Settings::sigAdjustBookmark, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(bool )>(_a, &Settings::sigShowCodeFlodFlag, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(bool )>(_a, &Settings::sigShowBlankCharacter, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(bool )>(_a, &Settings::sigHightLightCurrentLine, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(const QString & )>(_a, &Settings::sigThemeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(bool )>(_a, &Settings::sigSetLineNumberShow, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)(QString )>(_a, &Settings::sigChangeWindowSize, 10))
            return;
    }
}

const QMetaObject *Settings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Settings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SettingsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Settings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void Settings::sigAdjustFont(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Settings::sigAdjustFontSize(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Settings::sigAdjustTabSpaceNumber(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Settings::sigAdjustWordWrap(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Settings::sigAdjustBookmark(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Settings::sigShowCodeFlodFlag(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Settings::sigShowBlankCharacter(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Settings::sigHightLightCurrentLine(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Settings::sigThemeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Settings::sigSetLineNumberShow(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Settings::sigChangeWindowSize(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
