/****************************************************************************
** Meta object code from reading C++ file 'window.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../widgets/window.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'window.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6WindowE_t {};
} // unnamed namespace

template <> constexpr inline auto Window::qt_create_metaobjectdata<qt_meta_tag_ZN6WindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Window",
        "themeChanged",
        "",
        "themeName",
        "requestDragEnterEvent",
        "QDragEnterEvent*",
        "requestDropEvent",
        "QDropEvent*",
        "newWindow",
        "closeWindow",
        "sigJudgeBlockShutdown",
        "pressEsc",
        "tabChanged",
        "addBlankTab",
        "blankFile",
        "handleTabCloseRequested",
        "index",
        "handleTabsClosed",
        "tabList",
        "handleCurrentChanged",
        "handleJumpLineBarExit",
        "handleJumpLineBarJumpToLine",
        "filepath",
        "line",
        "focusEditor",
        "handleBackToPosition",
        "file",
        "row",
        "column",
        "scrollOffset",
        "handleFindNextSearchKeyword",
        "keyword",
        "handleFindPrevSearchKeyword",
        "handleFindKeyword",
        "state",
        "slotFindbarClose",
        "slotReplacebarClose",
        "slotSwitchToReplaceBar",
        "handleReplaceAll",
        "replaceText",
        "withText",
        "handleReplaceNext",
        "handleReplaceRest",
        "handleReplaceSkip",
        "handleRemoveSearchKeyword",
        "handleUpdateSearchKeyword",
        "QWidget*",
        "widget",
        "Qt::CaseSensitivity",
        "caseFlag",
        "loadTheme",
        "path",
        "showNewEditor",
        "EditWrapper*",
        "wrapper",
        "showNotify",
        "message",
        "warning",
        "getBlankFileIndex",
        "createDialog",
        "DDialog*",
        "title",
        "content",
        "slotLoadContentTheme",
        "DGuiApplicationHelper::ColorType",
        "themeType",
        "slotSettingResetTheme",
        "slot_saveReadingPath",
        "slot_beforeReplace",
        "_",
        "slot_setTitleFocus",
        "slotClearDoubleCharaterEncode",
        "slotSigThemeChanged",
        "slotSigAdjustFont",
        "fontName",
        "slotSigAdjustFontSize",
        "fontSize",
        "slotSigAdjustTabSpaceNumber",
        "number",
        "slotSigAdjustWordWrap",
        "enable",
        "slotSigSetLineNumberShow",
        "bIsShow",
        "slotSigAdjustBookmark",
        "slotSigShowBlankCharacter",
        "slotSigHightLightCurrentLine",
        "slotSigShowCodeFlodFlag",
        "slotSigChangeWindowSize",
        "mode",
        "updateSizeMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestDragEnterEvent'
        QtMocHelpers::SignalData<void(QDragEnterEvent *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'requestDropEvent'
        QtMocHelpers::SignalData<void(QDropEvent *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'newWindow'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeWindow'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sigJudgeBlockShutdown'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pressEsc'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tabChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addBlankTab'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addBlankTab'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'handleTabCloseRequested'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'handleTabsClosed'
        QtMocHelpers::SlotData<void(QStringList)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 18 },
        }}),
        // Slot 'handleCurrentChanged'
        QtMocHelpers::SlotData<void(const int &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'handleJumpLineBarExit'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleJumpLineBarJumpToLine'
        QtMocHelpers::SlotData<void(const QString &, int, bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 24 },
        }}),
        // Slot 'handleBackToPosition'
        QtMocHelpers::SlotData<void(const QString &, int, int, int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::Int, 27 }, { QMetaType::Int, 28 }, { QMetaType::Int, 29 },
        }}),
        // Slot 'handleFindNextSearchKeyword'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'handleFindPrevSearchKeyword'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'handleFindKeyword'
        QtMocHelpers::SlotData<void(const QString &, bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 }, { QMetaType::Bool, 34 },
        }}),
        // Slot 'slotFindbarClose'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotReplacebarClose'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSwitchToReplaceBar'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleReplaceAll'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 }, { QMetaType::QString, 40 },
        }}),
        // Slot 'handleReplaceNext'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::QString, 39 }, { QMetaType::QString, 40 },
        }}),
        // Slot 'handleReplaceRest'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 }, { QMetaType::QString, 40 },
        }}),
        // Slot 'handleReplaceSkip'
        QtMocHelpers::SlotData<void(QString, QString)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::QString, 31 },
        }}),
        // Slot 'handleRemoveSearchKeyword'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleUpdateSearchKeyword'
        QtMocHelpers::SlotData<void(QWidget *, const QString &, const QString &, Qt::CaseSensitivity)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { QMetaType::QString, 26 }, { QMetaType::QString, 31 }, { 0x80000000 | 48, 49 },
        }}),
        // Slot 'handleUpdateSearchKeyword'
        QtMocHelpers::SlotData<void(QWidget *, const QString &, const QString &)>(45, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { QMetaType::QString, 26 }, { QMetaType::QString, 31 },
        }}),
        // Slot 'loadTheme'
        QtMocHelpers::SlotData<void(const QString &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 },
        }}),
        // Slot 'showNewEditor'
        QtMocHelpers::SlotData<void(EditWrapper *)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'showNotify'
        QtMocHelpers::SlotData<void(const QString &, bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 56 }, { QMetaType::Bool, 57 },
        }}),
        // Slot 'showNotify'
        QtMocHelpers::SlotData<void(const QString &)>(55, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 56 },
        }}),
        // Slot 'getBlankFileIndex'
        QtMocHelpers::SlotData<int()>(58, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'createDialog'
        QtMocHelpers::SlotData<DDialog *(const QString &, const QString &)>(59, 2, QMC::AccessPublic, 0x80000000 | 60, {{
            { QMetaType::QString, 61 }, { QMetaType::QString, 62 },
        }}),
        // Slot 'slotLoadContentTheme'
        QtMocHelpers::SlotData<void(DGuiApplicationHelper::ColorType)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 65 },
        }}),
        // Slot 'slotSettingResetTheme'
        QtMocHelpers::SlotData<void(const QString &)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 },
        }}),
        // Slot 'slot_saveReadingPath'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slot_beforeReplace'
        QtMocHelpers::SlotData<void(QString)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 69 },
        }}),
        // Slot 'slot_setTitleFocus'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotClearDoubleCharaterEncode'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSigThemeChanged'
        QtMocHelpers::SlotData<void(const QString &)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 51 },
        }}),
        // Slot 'slotSigAdjustFont'
        QtMocHelpers::SlotData<void(QString)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 74 },
        }}),
        // Slot 'slotSigAdjustFontSize'
        QtMocHelpers::SlotData<void(qreal)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 76 },
        }}),
        // Slot 'slotSigAdjustTabSpaceNumber'
        QtMocHelpers::SlotData<void(int)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 78 },
        }}),
        // Slot 'slotSigAdjustWordWrap'
        QtMocHelpers::SlotData<void(bool)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 80 },
        }}),
        // Slot 'slotSigSetLineNumberShow'
        QtMocHelpers::SlotData<void(bool)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Slot 'slotSigAdjustBookmark'
        QtMocHelpers::SlotData<void(bool)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Slot 'slotSigShowBlankCharacter'
        QtMocHelpers::SlotData<void(bool)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Slot 'slotSigHightLightCurrentLine'
        QtMocHelpers::SlotData<void(bool)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Slot 'slotSigShowCodeFlodFlag'
        QtMocHelpers::SlotData<void(bool)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 82 },
        }}),
        // Slot 'slotSigChangeWindowSize'
        QtMocHelpers::SlotData<void(QString)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 88 },
        }}),
        // Slot 'updateSizeMode'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Window, qt_meta_tag_ZN6WindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Window::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6WindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6WindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6WindowE_t>.metaTypes,
    nullptr
} };

void Window::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Window *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->themeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->requestDragEnterEvent((*reinterpret_cast<std::add_pointer_t<QDragEnterEvent*>>(_a[1]))); break;
        case 2: _t->requestDropEvent((*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[1]))); break;
        case 3: _t->newWindow(); break;
        case 4: _t->closeWindow(); break;
        case 5: _t->sigJudgeBlockShutdown(); break;
        case 6: _t->pressEsc(); break;
        case 7: _t->tabChanged(); break;
        case 8: _t->addBlankTab(); break;
        case 9: _t->addBlankTab((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->handleTabCloseRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->handleTabsClosed((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 12: _t->handleCurrentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->handleJumpLineBarExit(); break;
        case 14: _t->handleJumpLineBarJumpToLine((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 15: _t->handleBackToPosition((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 16: _t->handleFindNextSearchKeyword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->handleFindPrevSearchKeyword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->handleFindKeyword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->slotFindbarClose(); break;
        case 20: _t->slotReplacebarClose(); break;
        case 21: _t->slotSwitchToReplaceBar(); break;
        case 22: _t->handleReplaceAll((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 23: _t->handleReplaceNext((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 24: _t->handleReplaceRest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 25: _t->handleReplaceSkip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 26: _t->handleRemoveSearchKeyword(); break;
        case 27: _t->handleUpdateSearchKeyword((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Qt::CaseSensitivity>>(_a[4]))); break;
        case 28: _t->handleUpdateSearchKeyword((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 29: _t->loadTheme((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->showNewEditor((*reinterpret_cast<std::add_pointer_t<EditWrapper*>>(_a[1]))); break;
        case 31: _t->showNotify((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 32: _t->showNotify((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: { int _r = _t->getBlankFileIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 34: { DDialog* _r = _t->createDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<DDialog**>(_a[0]) = std::move(_r); }  break;
        case 35: _t->slotLoadContentTheme((*reinterpret_cast<std::add_pointer_t<DGuiApplicationHelper::ColorType>>(_a[1]))); break;
        case 36: _t->slotSettingResetTheme((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->slot_saveReadingPath(); break;
        case 38: _t->slot_beforeReplace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->slot_setTitleFocus(); break;
        case 40: _t->slotClearDoubleCharaterEncode(); break;
        case 41: _t->slotSigThemeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->slotSigAdjustFont((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->slotSigAdjustFontSize((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 44: _t->slotSigAdjustTabSpaceNumber((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 45: _t->slotSigAdjustWordWrap((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->slotSigSetLineNumberShow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: _t->slotSigAdjustBookmark((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->slotSigShowBlankCharacter((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->slotSigHightLightCurrentLine((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->slotSigShowCodeFlodFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: _t->slotSigChangeWindowSize((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 52: _t->updateSizeMode(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EditWrapper* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Window::*)(const QString & )>(_a, &Window::themeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Window::*)(QDragEnterEvent * )>(_a, &Window::requestDragEnterEvent, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Window::*)(QDropEvent * )>(_a, &Window::requestDropEvent, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Window::*)()>(_a, &Window::newWindow, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Window::*)()>(_a, &Window::closeWindow, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Window::*)()>(_a, &Window::sigJudgeBlockShutdown, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Window::*)()>(_a, &Window::pressEsc, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Window::*)()>(_a, &Window::tabChanged, 7))
            return;
    }
}

const QMetaObject *Window::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Window::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6WindowE_t>.strings))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int Window::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 53)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 53;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 53)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 53;
    }
    return _id;
}

// SIGNAL 0
void Window::themeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Window::requestDragEnterEvent(QDragEnterEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Window::requestDropEvent(QDropEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Window::newWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Window::closeWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Window::sigJudgeBlockShutdown()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Window::pressEsc()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Window::tabChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
