/****************************************************************************
** Meta object code from reading C++ file 'findbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../controls/findbar.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'findbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7FindBarE_t {};
} // unnamed namespace

template <> constexpr inline auto FindBar::qt_create_metaobjectdata<qt_meta_tag_ZN7FindBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FindBar",
        "pressEsc",
        "",
        "findNext",
        "keyword",
        "findPrev",
        "removeSearchKeyword",
        "updateSearchKeyword",
        "file",
        "sigFindbarClose",
        "sigSwitchToReplaceBar",
        "findCancel",
        "handleContentChanged",
        "handleFindNext",
        "handleFindPrev",
        "handleSwitchToReplace",
        "updateSizeMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pressEsc'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'findNext'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'findPrev'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'removeSearchKeyword'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateSearchKeyword'
        QtMocHelpers::SignalData<void(QString, QString)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'sigFindbarClose'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sigSwitchToReplaceBar'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'findCancel'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleContentChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleFindNext'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleFindPrev'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSwitchToReplace'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSizeMode'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FindBar, qt_meta_tag_ZN7FindBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FindBar::staticMetaObject = { {
    QMetaObject::SuperData::link<DFloatingWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7FindBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7FindBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7FindBarE_t>.metaTypes,
    nullptr
} };

void FindBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FindBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pressEsc(); break;
        case 1: _t->findNext((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->findPrev((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->removeSearchKeyword(); break;
        case 4: _t->updateSearchKeyword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->sigFindbarClose(); break;
        case 6: _t->sigSwitchToReplaceBar(); break;
        case 7: _t->findCancel(); break;
        case 8: _t->handleContentChanged(); break;
        case 9: _t->handleFindNext(); break;
        case 10: _t->handleFindPrev(); break;
        case 11: _t->handleSwitchToReplace(); break;
        case 12: _t->updateSizeMode(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FindBar::*)()>(_a, &FindBar::pressEsc, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindBar::*)(const QString & )>(_a, &FindBar::findNext, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindBar::*)(const QString & )>(_a, &FindBar::findPrev, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindBar::*)()>(_a, &FindBar::removeSearchKeyword, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindBar::*)(QString , QString )>(_a, &FindBar::updateSearchKeyword, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindBar::*)()>(_a, &FindBar::sigFindbarClose, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindBar::*)()>(_a, &FindBar::sigSwitchToReplaceBar, 6))
            return;
    }
}

const QMetaObject *FindBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FindBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7FindBarE_t>.strings))
        return static_cast<void*>(this);
    return DFloatingWidget::qt_metacast(_clname);
}

int FindBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DFloatingWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void FindBar::pressEsc()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FindBar::findNext(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void FindBar::findPrev(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FindBar::removeSearchKeyword()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void FindBar::updateSearchKeyword(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void FindBar::sigFindbarClose()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void FindBar::sigSwitchToReplaceBar()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
