/****************************************************************************
** Meta object code from reading C++ file 'dtextedit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../editor/dtextedit.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dtextedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8TextEditE_t {};
} // unnamed namespace

template <> constexpr inline auto TextEdit::qt_create_metaobjectdata<qt_meta_tag_ZN8TextEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextEdit",
        "clickFindAction",
        "",
        "clickReplaceAction",
        "clickJumpLineAction",
        "clickFullscreenAction",
        "cursorMarkChanged",
        "mark",
        "QTextCursor",
        "cursor",
        "cursorModeChanged",
        "CursorMode",
        "mode",
        "hightlightChanged",
        "name",
        "popupNotify",
        "notify",
        "warning",
        "signal_readingPath",
        "signal_setTitleFocus",
        "addOrDeleteBookMark",
        "updateLeftAreaWidget",
        "handleScrollFinish",
        "setSyntaxDefinition",
        "KSyntaxHighlighting::Definition",
        "def",
        "slot_translate",
        "setHighLineCurrentLine",
        "ok",
        "upcaseWord",
        "downcaseWord",
        "capitalizeWord",
        "transposeChar",
        "handleCursorMarkChanged",
        "slotValueChanged",
        "iValue",
        "adjustScrollbarMargins",
        "slotSelectionChanged",
        "slotCanRedoChanged",
        "bCanRedo",
        "slotCanUndoChanged",
        "bCanUndo",
        "onSelectionArea",
        "fingerZoom",
        "direction",
        "fingers",
        "cursorPositionChanged",
        "cut",
        "ignoreCheck",
        "copy",
        "paste",
        "highlight",
        "selectTextInView",
        "setSelectAll",
        "slotSigColorSelected",
        "bSelected",
        "QColor",
        "color",
        "slotSigColorAllSelected",
        "slotCutAction",
        "checked",
        "slotCopyAction",
        "slotPasteAction",
        "slotDeleteAction",
        "slotSelectAllAction",
        "slotOpenInFileManagerAction",
        "slotAddComment",
        "slotCancelComment",
        "slotVoiceReadingAction",
        "slotStopReadingAction",
        "slotdictationAction",
        "onAudioPortEnabledChanged",
        "cardId",
        "portName",
        "enabled",
        "slotColumnEditAction",
        "slotPreBookMarkAction",
        "slotNextBookMarkAction",
        "slotClearBookMarkAction",
        "slotFlodAllLevel",
        "slotUnflodAllLevel",
        "slotFlodCurrentLevel",
        "slotUnflodCurrentLevel",
        "slotCancleMarkAllLine",
        "slotCancleLastMark",
        "slotUndoAvailable",
        "undoIsAvailable",
        "slotRedoAvailable",
        "redoIsAvailable",
        "redo_",
        "undo_",
        "moveText",
        "from",
        "to",
        "text",
        "findCursor",
        "substr",
        "backward",
        "cursorPos",
        "Qt::CaseSensitivity",
        "caseFlag",
        "onPressedLineNumber",
        "QPoint",
        "point",
        "selectedText",
        "checkCRLF",
        "onEndlineFormatChanged",
        "BottomBar::EndlineFormat",
        "onAppPaletteChanged",
        "onTextContentChanged",
        "charsRemoved",
        "charsAdded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clickFindAction'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clickReplaceAction'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clickJumpLineAction'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clickFullscreenAction'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorMarkChanged'
        QtMocHelpers::SignalData<void(bool, QTextCursor)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'cursorModeChanged'
        QtMocHelpers::SignalData<void(enum CursorMode)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'hightlightChanged'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'popupNotify'
        QtMocHelpers::SignalData<void(QString, bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::Bool, 17 },
        }}),
        // Signal 'popupNotify'
        QtMocHelpers::SignalData<void(QString)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'signal_readingPath'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signal_setTitleFocus'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addOrDeleteBookMark'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateLeftAreaWidget'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleScrollFinish'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSyntaxDefinition'
        QtMocHelpers::SlotData<void(KSyntaxHighlighting::Definition)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'slot_translate'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setHighLineCurrentLine'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'upcaseWord'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'downcaseWord'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'capitalizeWord'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transposeChar'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleCursorMarkChanged'
        QtMocHelpers::SlotData<void(bool, QTextCursor)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotValueChanged'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'adjustScrollbarMargins'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCanRedoChanged'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 39 },
        }}),
        // Slot 'slotCanUndoChanged'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 41 },
        }}),
        // Slot 'onSelectionArea'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fingerZoom'
        QtMocHelpers::SlotData<void(QString, QString, int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 44 }, { QMetaType::Int, 45 },
        }}),
        // Slot 'cursorPositionChanged'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cut'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Slot 'cut'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copy'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Slot 'copy'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'paste'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'highlight'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectTextInView'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSelectAll'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSigColorSelected'
        QtMocHelpers::SlotData<void(bool, QColor)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 55 }, { 0x80000000 | 56, 57 },
        }}),
        // Slot 'slotSigColorAllSelected'
        QtMocHelpers::SlotData<void(bool, QColor)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 55 }, { 0x80000000 | 56, 57 },
        }}),
        // Slot 'slotCutAction'
        QtMocHelpers::SlotData<void(bool)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotCutAction'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotCopyAction'
        QtMocHelpers::SlotData<void(bool)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotCopyAction'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotPasteAction'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotPasteAction'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotDeleteAction'
        QtMocHelpers::SlotData<void(bool)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotDeleteAction'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotSelectAllAction'
        QtMocHelpers::SlotData<void(bool)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotSelectAllAction'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotOpenInFileManagerAction'
        QtMocHelpers::SlotData<bool(bool)>(65, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotOpenInFileManagerAction'
        QtMocHelpers::SlotData<bool()>(65, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'slotAddComment'
        QtMocHelpers::SlotData<void(bool)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotAddComment'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotCancelComment'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotCancelComment'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotVoiceReadingAction'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotVoiceReadingAction'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotStopReadingAction'
        QtMocHelpers::SlotData<bool(bool)>(69, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotStopReadingAction'
        QtMocHelpers::SlotData<bool()>(69, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'slotdictationAction'
        QtMocHelpers::SlotData<void(bool)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotdictationAction'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onAudioPortEnabledChanged'
        QtMocHelpers::SlotData<void(quint32, const QString &, bool)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 72 }, { QMetaType::QString, 73 }, { QMetaType::Bool, 74 },
        }}),
        // Slot 'slotColumnEditAction'
        QtMocHelpers::SlotData<void(bool)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotColumnEditAction'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotPreBookMarkAction'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotPreBookMarkAction'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotNextBookMarkAction'
        QtMocHelpers::SlotData<void(bool)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotNextBookMarkAction'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotClearBookMarkAction'
        QtMocHelpers::SlotData<void(bool)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotClearBookMarkAction'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotFlodAllLevel'
        QtMocHelpers::SlotData<void(bool)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotFlodAllLevel'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotUnflodAllLevel'
        QtMocHelpers::SlotData<void(bool)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotUnflodAllLevel'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotFlodCurrentLevel'
        QtMocHelpers::SlotData<void(bool)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotFlodCurrentLevel'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotUnflodCurrentLevel'
        QtMocHelpers::SlotData<void(bool)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotUnflodCurrentLevel'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotCancleMarkAllLine'
        QtMocHelpers::SlotData<void(bool)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotCancleMarkAllLine'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotCancleLastMark'
        QtMocHelpers::SlotData<void(bool)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 60 },
        }}),
        // Slot 'slotCancleLastMark'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotUndoAvailable'
        QtMocHelpers::SlotData<void(bool)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 86 },
        }}),
        // Slot 'slotRedoAvailable'
        QtMocHelpers::SlotData<void(bool)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 88 },
        }}),
        // Slot 'redo_'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'undo_'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveText'
        QtMocHelpers::SlotData<void(int, int, const QString &, bool)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 92 }, { QMetaType::Int, 93 }, { QMetaType::QString, 94 }, { QMetaType::Bool, 49 },
        }}),
        // Slot 'moveText'
        QtMocHelpers::SlotData<void(int, int, const QString &)>(91, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 92 }, { QMetaType::Int, 93 }, { QMetaType::QString, 94 },
        }}),
        // Slot 'findCursor'
        QtMocHelpers::SlotData<QTextCursor(const QString &, const QString &, int, bool, int, Qt::CaseSensitivity)>(95, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 96 }, { QMetaType::QString, 94 }, { QMetaType::Int, 92 }, { QMetaType::Bool, 97 },
            { QMetaType::Int, 98 }, { 0x80000000 | 99, 100 },
        }}),
        // Slot 'findCursor'
        QtMocHelpers::SlotData<QTextCursor(const QString &, const QString &, int, bool, int)>(95, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 8, {{
            { QMetaType::QString, 96 }, { QMetaType::QString, 94 }, { QMetaType::Int, 92 }, { QMetaType::Bool, 97 },
            { QMetaType::Int, 98 },
        }}),
        // Slot 'findCursor'
        QtMocHelpers::SlotData<QTextCursor(const QString &, const QString &, int, bool)>(95, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 8, {{
            { QMetaType::QString, 96 }, { QMetaType::QString, 94 }, { QMetaType::Int, 92 }, { QMetaType::Bool, 97 },
        }}),
        // Slot 'findCursor'
        QtMocHelpers::SlotData<QTextCursor(const QString &, const QString &, int)>(95, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 8, {{
            { QMetaType::QString, 96 }, { QMetaType::QString, 94 }, { QMetaType::Int, 92 },
        }}),
        // Slot 'onPressedLineNumber'
        QtMocHelpers::SlotData<void(const QPoint &)>(101, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 102, 103 },
        }}),
        // Slot 'selectedText'
        QtMocHelpers::SlotData<QString(bool)>(104, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Bool, 105 },
        }}),
        // Slot 'selectedText'
        QtMocHelpers::SlotData<QString()>(104, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString),
        // Slot 'onEndlineFormatChanged'
        QtMocHelpers::SlotData<void(BottomBar::EndlineFormat, BottomBar::EndlineFormat)>(106, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 107, 92 }, { 0x80000000 | 107, 93 },
        }}),
        // Slot 'onAppPaletteChanged'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTextContentChanged'
        QtMocHelpers::SlotData<void(int, int, int)>(109, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 92 }, { QMetaType::Int, 110 }, { QMetaType::Int, 111 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextEdit, qt_meta_tag_ZN8TextEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<DPlainTextEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TextEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TextEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8TextEditE_t>.metaTypes,
    nullptr
} };

void TextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clickFindAction(); break;
        case 1: _t->clickReplaceAction(); break;
        case 2: _t->clickJumpLineAction(); break;
        case 3: _t->clickFullscreenAction(); break;
        case 4: _t->cursorMarkChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTextCursor>>(_a[2]))); break;
        case 5: _t->cursorModeChanged((*reinterpret_cast<std::add_pointer_t<enum CursorMode>>(_a[1]))); break;
        case 6: _t->hightlightChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->popupNotify((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->popupNotify((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->signal_readingPath(); break;
        case 10: _t->signal_setTitleFocus(); break;
        case 11: _t->addOrDeleteBookMark(); break;
        case 12: _t->updateLeftAreaWidget(); break;
        case 13: _t->handleScrollFinish(); break;
        case 14: _t->setSyntaxDefinition((*reinterpret_cast<std::add_pointer_t<KSyntaxHighlighting::Definition>>(_a[1]))); break;
        case 15: _t->slot_translate(); break;
        case 16: _t->setHighLineCurrentLine((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->upcaseWord(); break;
        case 18: _t->downcaseWord(); break;
        case 19: _t->capitalizeWord(); break;
        case 20: _t->transposeChar(); break;
        case 21: _t->handleCursorMarkChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTextCursor>>(_a[2]))); break;
        case 22: _t->slotValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->adjustScrollbarMargins(); break;
        case 24: _t->slotSelectionChanged(); break;
        case 25: _t->slotCanRedoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->slotCanUndoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->onSelectionArea(); break;
        case 28: _t->fingerZoom((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 29: _t->cursorPositionChanged(); break;
        case 30: _t->cut((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->cut(); break;
        case 32: _t->copy((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->copy(); break;
        case 34: _t->paste(); break;
        case 35: _t->highlight(); break;
        case 36: _t->selectTextInView(); break;
        case 37: _t->setSelectAll(); break;
        case 38: _t->slotSigColorSelected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 39: _t->slotSigColorAllSelected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 40: _t->slotCutAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->slotCutAction(); break;
        case 42: _t->slotCopyAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->slotCopyAction(); break;
        case 44: _t->slotPasteAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 45: _t->slotPasteAction(); break;
        case 46: _t->slotDeleteAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: _t->slotDeleteAction(); break;
        case 48: _t->slotSelectAllAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->slotSelectAllAction(); break;
        case 50: { bool _r = _t->slotOpenInFileManagerAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 51: { bool _r = _t->slotOpenInFileManagerAction();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 52: _t->slotAddComment((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: _t->slotAddComment(); break;
        case 54: _t->slotCancelComment((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 55: _t->slotCancelComment(); break;
        case 56: _t->slotVoiceReadingAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 57: _t->slotVoiceReadingAction(); break;
        case 58: { bool _r = _t->slotStopReadingAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 59: { bool _r = _t->slotStopReadingAction();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 60: _t->slotdictationAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 61: _t->slotdictationAction(); break;
        case 62: _t->onAudioPortEnabledChanged((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 63: _t->slotColumnEditAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 64: _t->slotColumnEditAction(); break;
        case 65: _t->slotPreBookMarkAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 66: _t->slotPreBookMarkAction(); break;
        case 67: _t->slotNextBookMarkAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 68: _t->slotNextBookMarkAction(); break;
        case 69: _t->slotClearBookMarkAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 70: _t->slotClearBookMarkAction(); break;
        case 71: _t->slotFlodAllLevel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 72: _t->slotFlodAllLevel(); break;
        case 73: _t->slotUnflodAllLevel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 74: _t->slotUnflodAllLevel(); break;
        case 75: _t->slotFlodCurrentLevel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 76: _t->slotFlodCurrentLevel(); break;
        case 77: _t->slotUnflodCurrentLevel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: _t->slotUnflodCurrentLevel(); break;
        case 79: _t->slotCancleMarkAllLine((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 80: _t->slotCancleMarkAllLine(); break;
        case 81: _t->slotCancleLastMark((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 82: _t->slotCancleLastMark(); break;
        case 83: _t->slotUndoAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 84: _t->slotRedoAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 85: _t->redo_(); break;
        case 86: _t->undo_(); break;
        case 87: _t->moveText((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 88: _t->moveText((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 89: { QTextCursor _r = _t->findCursor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Qt::CaseSensitivity>>(_a[6])));
            if (_a[0]) *reinterpret_cast<QTextCursor*>(_a[0]) = std::move(_r); }  break;
        case 90: { QTextCursor _r = _t->findCursor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QTextCursor*>(_a[0]) = std::move(_r); }  break;
        case 91: { QTextCursor _r = _t->findCursor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QTextCursor*>(_a[0]) = std::move(_r); }  break;
        case 92: { QTextCursor _r = _t->findCursor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QTextCursor*>(_a[0]) = std::move(_r); }  break;
        case 93: _t->onPressedLineNumber((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 94: { QString _r = _t->selectedText((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 95: { QString _r = _t->selectedText();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 96: _t->onEndlineFormatChanged((*reinterpret_cast<std::add_pointer_t<BottomBar::EndlineFormat>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<BottomBar::EndlineFormat>>(_a[2]))); break;
        case 97: _t->onAppPaletteChanged(); break;
        case 98: _t->onTextContentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)()>(_a, &TextEdit::clickFindAction, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)()>(_a, &TextEdit::clickReplaceAction, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)()>(_a, &TextEdit::clickJumpLineAction, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)()>(_a, &TextEdit::clickFullscreenAction, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)(bool , QTextCursor )>(_a, &TextEdit::cursorMarkChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)(CursorMode )>(_a, &TextEdit::cursorModeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)(const QString & )>(_a, &TextEdit::hightlightChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)(QString , bool )>(_a, &TextEdit::popupNotify, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)()>(_a, &TextEdit::signal_readingPath, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextEdit::*)()>(_a, &TextEdit::signal_setTitleFocus, 10))
            return;
    }
}

const QMetaObject *TextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TextEditE_t>.strings))
        return static_cast<void*>(this);
    return DPlainTextEdit::qt_metacast(_clname);
}

int TextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DPlainTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 99)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 99;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 99)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 99;
    }
    return _id;
}

// SIGNAL 0
void TextEdit::clickFindAction()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextEdit::clickReplaceAction()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextEdit::clickJumpLineAction()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TextEdit::clickFullscreenAction()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TextEdit::cursorMarkChanged(bool _t1, QTextCursor _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void TextEdit::cursorModeChanged(CursorMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TextEdit::hightlightChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TextEdit::popupNotify(QString _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 9
void TextEdit::signal_readingPath()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TextEdit::signal_setTitleFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
